function convgraphs(flist)
% script to produce convergence graphs comparing different functions
% prints just the median of bestever for every case
dim = 30;

% totrun 
totrun = 25;
ter_err = 1e-8;
maxpoints = dim*10000;
styles  = {'b-','r--','g-.','c-','m--','k-.'};
sstyles = {'bs','ro','g+','c^','mx','kv'};
ifig = flist(1);
close all;
figure(ifig);
h_bestever1 = [];
h_bestever2 = [];
maxf = 0;
minf = inf;
mineval = 0;
maxeval = 0;
legendstr = '';

for k=1:length(flist)
    dir = strcat('fun',num2str(flist(k)));
    eval(['cd ',dir]);
    load('fbias_data');
    filename = strcat('bestever_f',num2str(flist(k)),'_',num2str(dim),'D');

    % load data 
    load(filename);

    % reconstruct full data
    fulldata = zeros(maxpoints,totrun);

    imax = size(bestever,1);
    run   = 0;
    for i=1:imax
        jj  = bestever(i,1);
        fj  = bestever(i,2);
        if jj==1 
            run = run+1;
            if run>totrun
                warning('too many runs in file!');
                break;
            end
            fulldata(jj,run)=fj-f_bias(flist(k));
            if run>1
                % fill up the preceeding column
                fulldata(jlast+1:maxpoints,run-1)=fulldata(jlast,run-1);
            end
            jlast = 1; 
        elseif isnan(jj)
            fulldata(jlast+1:maxpoints,run)=fulldata(jlast,run);
            break;   
        end
        if jj>maxpoints*1.1
            warning('possibly corrupt point, skip');
            run
            jj
            jlast
        else
            fulldata(jj,run) = fj-f_bias(flist(k));
    
            for j=jj-1:-1:jlast+1        
                %fulldata(j,run) = fj-f_bias(nfun);
                fulldata(j,run) = fjlast-f_bias(flist(k));
            end    
            jlast = jj;  
            fjlast = fj;
        end
    end
    fulldata(jlast+1:maxpoints,run)=fulldata(jlast,run);
    
    if nrun~=totrun
        disp('wrong number of runs in file!');
        disp(nrun);
    end
    % clear unused stuff
    clear bestever;

    % plot of median run
    imax = 1;
    for i=1:totrun    
        i_ter = min(find(fulldata(:,i)<ter_err));
        if  i_ter > imax 
            imax = i_ter;
        elseif ~isscalar(i_ter);
            imax = maxpoints;
            break;
        end
    end
    
    convgraph = zeros(imax,2);
    for i=1:imax
        convgraph(i,1)=i;
        sortedline = sort(fulldata(i,:));    
        if sortedline(13) > ter_err
            convgraph(i,2)=sortedline(13);
        else
            convgraph(i,2)=NaN;
        end
    end
    
    % find min max f, neval
    if convgraph(imax,1) > maxeval
        maxeval = convgraph(imax,1);
    end
    minftmp = min(convgraph(:,2));
    if minftmp < minf
        minf = minftmp;
    end
    if convgraph(1,2) > maxf
        maxf = convgraph(1,2);
    end
        
    freq = round(imax/5);    
    h_bestever1 = [h_bestever1, semilogy(convgraph(:,1),convgraph(:,2),styles{k})];
    hold on;
    h_bestever2 = [h_bestever2, semilogy(convgraph(round(freq/2):freq:end,1),convgraph(round(freq/2):freq:end,2),sstyles{k})];
    hold on;
    eval(['cd ../']);
    legendstr = strcat(legendstr,',num2str(flist(',num2str(k),'))');    
end
set(h_bestever1,'LineWidth',2);
set(h_bestever2,'MarkerSize',10);
set(h_bestever2,'LineWidth',2);
grid on;
set(gca,'FontSize',14);
xlabel('function evaluations');
ylabel('f');
axis([mineval,maxeval,minf,maxf]);
hold off;
drawnow;
filename = strcat('convergence_f',num2str(flist(1)),'-f',num2str(flist(end)),'_',num2str(dim),'D.eps');
eval(['print -f',num2str(ifig),' -depsc2 ',filename]);
eval(['legend(h_bestever2',legendstr,');']);
eval(['print -f',num2str(ifig),' -depsc2 ',strcat('leg_',filename)]);
