function postcec(nfun,dim)
%----------------------------------------------------------------
% Postprocessing script for the recorded data for the CEC special 
% session. The script is provided as is without any warranty.
%
% the bestever statistics are per function evaluation
% the generational data is sorted according to the
% bestever performance
%-----------------------------------------------------------------

filename1 = strcat('bestever_f',num2str(nfun),'_',num2str(dim),'D');
filename2 = strcat('bestgen_f',num2str(nfun),'_',num2str(dim),'D');

load(filename1);
load('fbias_data');

% success levels
success_levels = [1e-6,1e-6,1e-6,1e-6,...
    1e-6,1e-2,1e-2,1e-2,1e-2,...
    1e-2,1e-2,1e-2,1e-2,1e-2,...
    1e-2,1e-2,1e-2,1e-1,1e-1,1e-1,...
    1e-1,1e-1,1e-1,1e-1,1e-1];
    
% totrun 
totrun = 25;
ter_err = 1e-8;

maxpoints = dim*10000;
fulldata = zeros(maxpoints,totrun);
poi = [1000,10000,100000,maxpoints];
pack;
% reconstruct the whole data
imax = size(bestever,1);
run   = 0;
for i=1:imax
    jj  = bestever(i,1);
    fj  = bestever(i,2);
    if jj==1 
        run = run+1;
        if run>totrun
            warning('too many runs in file!');
            break;
        end
        fulldata(jj,run)=fj-f_bias(nfun);
        if run>1
            % fill up the preceeding column
            fulldata(jlast+1:maxpoints,run-1)=fulldata(jlast,run-1);
        end
        jlast = 1; 
    elseif isnan(jj)
        fulldata(jlast+1:maxpoints,run)=fulldata(jlast,run);
        break;   
    end
    if jj>maxpoints*1.1
        warning('possibly corrupt point, skip');
        run
        jj
        jlast
    else
        fulldata(jj,run) = fj-f_bias(nfun);
    
        for j=jj-1:-1:jlast+1        
            %fulldata(j,run) = fj-f_bias(nfun);
            fulldata(j,run) = fjlast-f_bias(nfun);
        end    
        jlast = jj;  
        fjlast = fj;
    end
end
fulldata(jlast+1:maxpoints,run)=fulldata(jlast,run);
%fulldata(end,:)
if nrun~=totrun
    disp('wrong number of runs in file!');
    disp(nrun);
end
% clear unused data
clear bestever;
%%%%%%%%%%%
% reconstruct generational data
load(filename2);
fulldataG  = zeros(maxpoints,totrun);
pack;

%
imax = size(bestgen,1);
run   = 0;
for i=1:imax
    jj  = bestgen(i,1);
    fj  = bestgen(i,2);
    if jj==1 
        run = run+1;
        fulldataG(jj,run)=fj-f_bias(nfun);
        if run>1
            % fill up the preceeding column
            fulldataG(jlast+1:maxpoints,run-1)=fulldataG(jlast,run-1);
        end        
        jlast = 1;
        fjlast = fj;
    elseif isnan(jj)
        fulldataG(jlast+1:maxpoints,run)=fulldataG(jlast,run);
        break;
    end 
    fulldataG(jj,run)=fj-f_bias(nfun);
    for j=jj-1:-1:jlast+1        
        %fulldataG(j,run) = fj-f_bias(nfun);
        fulldataG(j,run) = fjlast-f_bias(nfun);
    end        
    jlast = jj;
    fjlast = fj;
end
%fill up last column:
fulldataG(jlast+1:maxpoints,run)=fulldataG(jlast,run);
%fulldataG(end,:)
if nrun~=totrun
    disp('wrong number of runs in bestgen file!');
    disp(nrun);
end
%%%%%%%%%%%
% clear unused stuff and reload fulldata
clear bestgen;
%%%%%%%%%%%%%%
table = fulldata(poi,:);
table = sort(table,2);

postdata = [poi',table(:,[1,7,13,19,25]),mean(table,2),std(table,0,2)];
fileout  = strcat('fvalues_f',num2str(nfun),'_',num2str(dim),'D.dat');
eval(['save -ascii ',fileout, ' postdata']);

% number of function evaluations, success rate, and 
% table to sort generational data

fsuc = success_levels(nfun);
nevalsuc = zeros(1,totrun);
nsuccess = 0;
for i=1:totrun    
    ns = min(find(fulldata(:,i)<fsuc));
    if  (isscalar(ns)) & (ns<=maxpoints)
        nevalsuc(i) = ns;
        nsuccess = nsuccess+1;   
    else
        nevalsuc(i) = NaN;
    end
end

performance_table = zeros(totrun,2);
for i=1:totrun    
    ns = min(find(fulldata(:,i)<ter_err));
    if  (isscalar(ns)) & (ns<=maxpoints)
        performance_table(i,1)=ns;
        performance_table(i,2)=fulldata(ns,i);
    else
        performance_table(i,1)=maxpoints;
        performance_table(i,2)=fulldata(maxpoints,i);
    end
end

% extract the min max and median run
[dummy,ind_perf]=sortrows(performance_table,[1,2]);
mmmrun = ind_perf([1,13,25])';

nevalsuc = sort(nevalsuc);
if nsuccess > 0
    successrate = nsuccess/totrun;
else
    successrate = NaN;
end
fprintf('success rate = %8.6f\n',successrate);

% do the statistics only from the successful runs
iend = max(find(~isnan(nevalsuc)));
if ~isscalar(iend)
    iend = 0;
end

performance = [nevalsuc([1,7,13,19,25]),mean(nevalsuc(1:iend)),std(nevalsuc(1:iend))];
performance = [performance; performance/successrate];
fileout = strcat('performance_f',num2str(nfun),'_',num2str(dim),'D.dat');
eval(['save -ascii ',fileout, ' successrate performance']);

% plot of median run
imax = 1;
for i=1:totrun    
    i_ter = min(find(fulldata(:,i)<ter_err));
    if  i_ter > imax 
        imax = i_ter;
    elseif ~isscalar(i_ter);
        imax = maxpoints;
        break;
    end
end

convgraph = zeros(imax,7);
for i=1:imax
    convgraph(i,1)=i;
    sortedline = sort(fulldata(i,:));
    sortedgen  = fulldataG(i,ind_perf');
    if sortedline(1) > ter_err
        convgraph(i,2)=sortedline(1);
    else
        convgraph(i,2)=NaN;
    end
     if sortedline(13) > ter_err
        convgraph(i,3)=sortedline(13);
    else
        convgraph(i,3)=NaN;
    end
    if sortedline(25) > ter_err
        convgraph(i,4)=sortedline(25);
    else
        convgraph(i,4)=NaN;
    end
    %%%
    if sortedgen(1) > ter_err
        convgraph(i,5)=sortedgen(1);
    else
        convgraph(i,5)=NaN;
    end
     if sortedgen(13) > ter_err
        convgraph(i,6)=sortedgen(13);
    else
        convgraph(i,6)=NaN;
    end
    if sortedgen(25) > ter_err
        convgraph(i,7)=sortedgen(25);
    else
        convgraph(i,7)=NaN;
    end    
end

% generation data 


% plot figure

figure(nfun);
clf;
h_bestgen  = [];
h_bestever = [];
h_bestever = [h_bestever, semilogy(convgraph(:,1),convgraph(:,2),'-b')];
hold on;
h_bestever = [h_bestever, semilogy(convgraph(:,1),convgraph(:,3),'-b')];
h_bestever = [h_bestever, semilogy(convgraph(:,1),convgraph(:,4),'-b')];
h_bestgen  = [h_bestgen, semilogy(convgraph(:,1),convgraph(:,5),'-r')];
h_bestgen  = [h_bestgen, semilogy(convgraph(:,1),convgraph(:,6),'-r')];
h_bestgen  = [h_bestgen, semilogy(convgraph(:,1),convgraph(:,7),'-r')];
set(h_bestever,'LineWidth',2);
grid on;
set(gca,'FontSize',14);
xlabel('function evaluations');
ylabel('f');
titlestring = sprintf('function %d, %dD',nfun,dim);
title(titlestring);
drawnow;
filename = strcat('convergence_f',num2str(nfun),'_',num2str(dim),'D.eps');
eval(['print -f',num2str(nfun),' -depsc2 ',filename]);

% save the data
filename = strcat('convdata_f',num2str(nfun),'_',num2str(dim),'D.mat');
eval(['save ',filename,' convgraph']);
    

